AC_DEFUN([XORGSERV_XSERVER],
  [
    AC_REQUIRE([PKG_PROG_PKG_CONFIG])
    AC_ARG_WITH([system-xserver],
      [AC_HELP_STRING([--with-system-xserver], 
         [Use the system xserver instead @<:@default=$SYS_XSERVER@:>@])],
      [ if test "$withval" != "no" ; then
          AC_MSG_CHECKING([for system Xserver])
          if $PKG_CONFIG --exists xorg-server ;then
            SYS_XSERVER=yes
          else
            SYS_XSERVER=no
          fi
          AC_MSG_RESULT([$SYS_XSERVER])
        else
          SYS_XSERVER=no
        fi ],[SYS_XSERVER=no])
    if test "$SYS_XSERVER" = "no" ; then
      AC_CONFIG_SUBDIRS([xorg-server])

      AC_ARG_ENABLE(xgl, AS_HELP_STRING([--enable-xgl],
        [Build Xgl server (default: no)]), [XGL=$enableval], [XGL=no])

      AC_ARG_ENABLE(xorg, AS_HELP_STRING([--enable-xorg],
        [Build Xorg server (default: auto)]), [XORG=$enableval], [XORG=auto])

      if test "$XORG" = "auto" ; then
        case $host_os in
          cygwin*) XORG="no" ;;
          darwin*) XORG="no" ;;
          *)       XORG="yes" ;;
        esac
      fi

      XORG_EXTRA_CFLAGS="-I\$(top_builddir)/xorg-server/include \
                   -I\$(top_srcdir)/xorg-server/include \
                   -I\$(top_srcdir)/xorg-server/Xext \
                   -I\$(top_srcdir)/xorg-server/mfb \
                   -I\$(top_srcdir)/xorg-server/render \
                   -I\$(top_srcdir)/xorg-server/fb \
                   -I\$(top_srcdir)/xorg-server/exa \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/common \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/vgahw \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/fbdevhw \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/xf1bpp \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/xf4bpp \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/xf8_16bpp \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/xaa \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/ramdac \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/i2c \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/ddc \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/shadowfb \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/int10 \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/rac \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/vbe \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/os-support \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/os-support/bus \
                   -I\$(top_srcdir)/xorg-server/hw/xfree86/dixmods/extmod \
                   -I\$(top_srcdir)/xorg-server/miext/shadow \
                   -I\$(top_srcdir)/xorg-server/miext/damage \
                   -I\$(top_srcdir)/xorg-server/mi "
      if test "$XORG" = "yes" -o "$XGL" = "yes" ; then
        XORG_EXTRA_CFLAGS="$XORG_EXTRA_CFLAGS \
                   -DIN_MODULE -DXFree86Module -DXFree86LOADER -DXFree86Server"
      fi
      XORG_EXTRA_LIBS=""
      XORG_SUBDIRS=xorg-server
      sdkdir="${includedir}/xorg"
      def_moduledir="${libdir}/xorg/modules"
      XORG_ENABLE_MODULE([xorg-server])
    else
      if $PKG_CONFIG --exists "xorg-server >= $1" ; then
        XORG_EXTRA_CFLAGS=$($PKG_CONFIG --cflags xorg-server)
        XORG_EXTRA_LIBS=$($PKG_CONFIG --libs xorg-server)
        sdkdir=$($PKG_CONFIG --variable=sdkdir xorg-server)
        def_moduledir=$($PKG_CONFIG --variable=moduledir xorg-server)
      else
        AC_MSG_ERROR([Xserver is too old for these drivers])
      fi
      XORG_ENABLE_MODULE([xorg-server], [sys])
    fi
    AM_CONDITIONAL([BUILD_XSERVER], [test "$SYS_XSERVER" = "no"])
    AC_SUBST([XORG_SUBDIRS])

    AC_ARG_WITH(module-dir, AS_HELP_STRING([--with-module-dir=DIR],
      [Directory where modules are installed (default: $libdir/xorg/modules)]),
      [ moduledir="$withval" ], [ moduledir="$def_moduledir" ])

    AC_SUBST([XORG_CFLAGS])
    AC_SUBST([XORG_LIBS])
    AC_SUBST([moduledir])
    AC_SUBST([sdkdir])
    AC_SUBST([inputdir], [$moduledir/input])
    AC_CONFIG_COMMANDS_PRE([XORG_OUTPUT_RESULTS([server], [xorg-server])])
  ])

AC_DEFUN([XORGSERV_DRIVERS],
  [
    DEFAULT_XORGXSERV_DRIVERS="$2"
    XORGXSERV_DRIVERS=""
    AC_ARG_ENABLE([$1-drivers], [AC_HELP_STRING([--enable-$1-drivers="list"],
      [List of $1 drivers to build @<:@default=ALL@:>@ available drivers are - @<:@$2@:>@])],
      [
        case "$enableval" in
          no|NO|none|NONE)  XORGXSERV_DRIVERS="" ;;
          yes|YES|all|ALL)  XORGXSERV_DRIVERS="$DEFAULT_XORGXSERV_DRIVERS" ;;
          *)    XORGXSERV_DRIVERS="`echo $enableval | sed -n 's/,/ /g ; p'`" ;;
        esac
      ],
      [XORGXSERV_DRIVERS="$DEFAULT_XORGXSERV_DRIVERS"])
    for Driver in $XORGXSERV_DRIVERS ; do
      if eval "test \"x\$XBUILD_$1_$Driver\" != \"xno\"" ; then
        eval XBUILD_$1_$Driver=yes
      fi
    done
    for Driver in $DEFAULT_XORGXSERV_DRIVERS ; do
      if eval "test \"x\$XBUILD_$1_$Driver\" != \"xyes\"" ; then
        eval XBUILD_$1_$Driver=no
      fi
    done
    AC_CONFIG_COMMANDS_PRE([
      XORG_M4_FOR([XORGXSERV_CHECK([$1],[@1],[$Capitols([$1_MODULES])])],[$2])dnl
      XORG_OUTPUT_RESULTS([$1], XORG_PFX_LST([$2], [$1-]))
     ])
  ])

AC_DEFUN([XORGXSERV_CHECK],
  [
    AC_REQUIRE([PKG_PROG_PKG_CONFIG])
    XORGXSERV_MOD="$3 $Capitols([$1_$2_EXTRA_MODULES])"
    if test "x$XBUILD_$1_$2" != "xno" ; then
      if test "$SYS_XSERVER" = "yes" ; then
        XORGXSERV_MOD="$XORGXSERV_MOD xorg-server"
      fi
      if test "x$XORGXSERV_MOD" != "x" ; then
        AC_MSG_CHECKING([[for $1-$2 deps - [$XORGXSERV_MOD] ]])
        if $PKG_CONFIG --exists "$XORGXSERV_MOD" ; then
          Capitols([$2_CFLAGS])="$($PKG_CONFIG --cflags $XORGXSERV_MOD) $XORG_EXTRA_CFLAGS $Capitols([$1_$2_EXTRA_CFLAGS])"
          Capitols([$2_LIBS])="$($PKG_CONFIG --libs $XORGXSERV_MOD) $XORG_EXTRA_LIBS $Capitols([$1_$2_EXTRA_LIBS])"
        else
          XBUILD_$1_$2="no"
        fi
      else
        AC_MSG_CHECKING([for $1-$2 deps ])
      fi
      Capitols([AC_SUBST([$2_CFLAGS])])dnl
      Capitols([AC_SUBST([$2_LIBS])])dnl
      AC_MSG_RESULT([$XBUILD_$1_$2])
    fi
    Capitols([AC_SUBST([$2_CCASFLAGS],["$$1_$2_EXTRA_CCASFLAGS"])])dnl
    Capitols([AC_SUBST([$2_CCASLIBS],["$$1_$2_EXTRA_CCASLIBS"])])dnl
    if test "x$XBUILD_$1_$2" != "xno" ; then
      XORG_SUBDIRS="$XORG_SUBDIRS $1-$2"
    fi
  ])


AC_DEFUN([XORGXSERV_DRIVER_CHECK],
  [
    AC_MSG_CHECKING([if $1 is defined])
    AS_IF([test "$SYS_XSERVER" = "yes"],
    [
      SAVE_CFLAGS="$CFLAGS"
      CFLAGS="$CFLAGS -I$($PKG_CONFIG --variable=sdkdir xorg-server)"
      AC_COMPILE_IFELSE([AC_LANG_PROGRAM([[
#include "xorg-server.h"
#if !defined $1
#error $1 not defined
#endif
      ]])],
      [DRV_$1_CHECK=yes],
      [DRV_$1_CHECK=no])
      CFLAGS="$SAVE_CFLAGS"
    ],[ifelse([$2],[],[DRV_$1_CHECK=yes],[
       if $2 ; then
         DRV_$1_CHECK=yes
       else
         DRV_$1_CHECK=no
       fi])])
    AC_MSG_RESULT([$DRV_$1_CHECK])
    AS_IF([test "$DRV_$1_CHECK" = "yes"],[$3])
  ])

AC_DEFUN([XORGXSERV_ENABLE_DRIVER],dnl
  [dnl
    XBUILD_$1_$2=yes
  ])dnl

AC_DEFUN([XORGXSERV_DISABLE_DRIVER],dnl
  [dnl
    XBUILD_$1_$2=no
  ])dnl

AC_DEFUN([XORGXSERV_DRIVER_FLAGS],dnl
  [dnl
    Capitols([$1_$2_EXTRA_$3])="$Capitols([$1_$2_EXTRA_$3]) $4"
  ])dnl

AC_DEFUN([XORGXSERV_DRIVER_XORG_CFLAGS],dnl
  [dnl
    Capitols([$1_$2_EXTRA_$3])="$Capitols([$1_$2_EXTRA_$3]) $XORG_EXTRA_CFLAGS"
  ])dnl

AC_DEFUN([XORGXSERV_DRI],
  [
    AC_REQUIRE([AC_CANONICAL_HOST])
    AC_REQUIRE([PKG_PROG_PKG_CONFIG])

    AC_ARG_ENABLE(dri, AC_HELP_STRING([--disable-dri],
      [Disable DRI support [[default=auto]]]), [DRI="$enableval"], [DRI="auto"])

    if test "$DRI" != no; then
      if test "$SYS_XSERVER" = "no" ; then
        AC_CHECK_FILE([${srcdir}/xorg-server/hw/xfree86/dri/dri.h],
          [have_dri_h="yes"], [have_dri_h="no"])
        AC_CHECK_FILE([${srcdir}/xorg-server/hw/xfree86/dri/sarea.h],
          [have_sarea_h="yes"], [have_sarea_h="no"])
        AC_CHECK_FILE([${srcdir}/xorg-server/hw/xfree86/dri/dristruct.h],
          [have_dristruct_h="yes"], [have_dristruct_h="no"])
        DRI_CFLAGS="$DRI_CFLAGS -I\$(top_srcdir)/xorg-server/hw/xfree86/dri \
                    -I\$(top_srcdir)/xorg-server/hw/xfree86/xf8_32bpp \
                    -I\$(top_srcdir)/xorg-server/randr \
                    -I\$(top_srcdir)/xorg-server/cfb"
      else
        AC_CHECK_FILE([${sdkdir}/dri.h],
          [have_dri_h="yes"], [have_dri_h="no"])
        AC_CHECK_FILE([${sdkdir}/sarea.h],
          [have_sarea_h="yes"], [have_sarea_h="no"])
        AC_CHECK_FILE([${sdkdir}/dristruct.h],
          [have_dristruct_h="yes"], [have_dristruct_h="no"])
        DRI_CFLAGS="$DRI_CFLAGS -I${sdkdir}"
      fi
    fi

    if test "$DRI" = "auto" ; then
      case "$host_os" in
         *freebsd*|*netbsd*|*linux*) if test "$have_dri_h" = "yes" -a \
                                             "$have_sarea_h" = "yes" -a \
                                             "$have_dristruct_h" = "yes" ; then
                                       DRI=yes
                                     else
                                       DRI=no
                                     fi ;;
         *)                          DRI=no  ;;
      esac
    fi

    if test "$DRI" != "no" ; then
      if $PKG_CONFIG --exists "libdrm >= 2.0 xf86driproto"
        then
          DRI_CFLAGS="$DRI_CFLAGS $($PKG_CONFIG --cflags libdrm xf86driproto)"
          DRI_LIBS=$($PKG_CONFIG --libs libdrm xf86driproto)
          AC_DEFINE(XF86DRI,1,[Enable DRI driver support])
          AC_DEFINE(XF86DRI_DEVEL,1,[Enable developmental DRI driver support])
          case "$host_cpu" in
            i*86|amd64*|x86_64*) XVMC=yes ;;
            *)                   XVMC=no ;;
          esac
        else
          DRI=no
          XVMC=no
        fi
    fi
    if test "x$XVMC" = xyes; then
      AC_CHECK_HEADERS(pthread.h sys/ioctl.h sys/time.h time.h,,[XVMC="no"; break],)
    fi
    AC_MSG_CHECKING([whether to include DRI support])
    AC_MSG_RESULT([$DRI])

    AC_SUBST([DRI_CFLAGS])
    AC_SUBST([DRI_LIBS])
    AM_CONDITIONAL(DRI, test "x$DRI" = "xyes")

    AC_MSG_CHECKING([whether to build via XvMC driver support])
    AC_MSG_RESULT([$XVMC])
    AM_CONDITIONAL(XVMC, test x$XVMC = xyes)
  ])

AC_DEFUN([XORGXSERV_ATI],
  [
    AC_REQUIRE([AC_CANONICAL_HOST])
    AC_REQUIRE([PKG_PROG_PKG_CONFIG])

    if test "$XBUILD_video_ati" != "no" ; then
      ATIMISC_CPIO=no
      ATIMISC_NON_PCI=yes
      ATIMISC_DGA=yes
      ATIMISC_TV_OUT=no

      case $host_cpu in
        i*86)               ATIMISC_TV_OUT=yes
                            ATIMISC_CPIO=yes ;;
        x86_64|amd64|alpha) ATIMISC_CPIO=yes ;;
        sparc)              ATIMISC_DGA=no ;;
        powerpc)            ATIMISC_NON_PCI=no ;;
       *)                   ;;
      esac
    
      AC_MSG_CHECKING([whether to include PIO support])
      if test "x$ATIMISC_CPIO" = xyes; then
        AC_MSG_RESULT([yes, PIO])
      else
        AC_DEFINE(AVOID_CPIO, 1, [Avoid PIO and use MMIO for atimisc.])
        AC_MSG_RESULT([no, MMIO])
      fi

      AC_MSG_CHECKING([whether to include support for non-PCI devices])
#      if test "x$ATI_AVOID_NON_PCI" = xyes; then
      if test "x$ATIMISC_NON_PCI" = xyes; then
        AC_MSG_RESULT([yes, PCI and ISA])
      else
        AC_DEFINE(AVOID_NON_PCI, 1, [Only probe PCI cards; do not probe ISA.])
        AC_MSG_RESULT([no, PCI only])
      fi

      AC_MSG_CHECKING([whether to include DGA support])
      if ! test "x$ATIMISC_DGA" = xyes; then
        AC_DEFINE(AVOID_DGA, 1, [Do not build DGA support.])
      fi
      AC_MSG_RESULT([$ATIMISC_DGA])

      AC_MSG_CHECKING([whether to include TV Out support])
      if test "x$ATIMISC_TV_OUT" = xyes; then
        AC_DEFINE(TV_OUT, 1, [Build TV-Out support for atimisc.])
      fi
      AC_MSG_RESULT([$ATIMISC_TV_OUT])

      AC_DEFINE(USE_XAA, 1, [Build support for XAA])

# Properly handle EXA.
      AC_MSG_CHECKING([whether to enable EXA support])
      if test "x$EXA" = xyes; then
        AC_MSG_RESULT([yes])
        if test "$SYS_XSERVER" = "no" ; then
          AC_CHECK_FILE([${srcdir}/xorg-server/exa/exa.h],
            [have_exa_h="yes"], [have_exa_h="no"])
        else
          AC_CHECK_FILE([${sdkdir}/exa.h],
            [have_exa_h="yes"], [have_exa_h="no"])
        fi
      else
        AC_MSG_RESULT([no])
      fi

      if test "x$have_exa_h" = xyes; then
        AC_MSG_CHECKING([wether EXA version is at least 2.0.0])
        SAVE_CPPFLAGS="$CPPFLAGS"
        if test "$SYS_XSERVER" = "no" ; then
          CPPFLAGS="$($PKG_CONFIG --cflags xproto) -Ixorg-server/include -Ixorg-server/render"
          AC_PREPROC_IFELSE([AC_LANG_PROGRAM([[
#include "xorg-server/exa/exa.h"
#if EXA_VERSION_MAJOR < 2
#error OLD EXA!
#endif
            ]])], [USE_EXA=yes], [USE_EXA=no])
        else
          CPPFLAGS="$XORG_EXTRA_CFLAGS"
          AC_PREPROC_IFELSE([AC_LANG_PROGRAM([[
#include "${sdkdir}/exa.h"
#if EXA_VERSION_MAJOR < 2
#error OLD EXA!
#endif
            ]])], [USE_EXA=yes], [USE_EXA=no])
        fi
        CPPFLAGS="$SAVE_CPPFLAGS"
        if test "x$USE_EXA" = xyes; then
          AC_DEFINE(USE_EXA, 1, [Build support for Exa])
        fi
        AC_MSG_RESULT([$USE_EXA])
      fi
    fi
    AM_CONDITIONAL(ATIMISC_CPIO, test "x$ATIMISC_CPIO" = xyes)
    AM_CONDITIONAL(ATIMISC_NON_PCI, test "x$ATIMISC_NON_PCI" = xyes)
    AM_CONDITIONAL(ATIMISC_DGA, test "x$ATIMISC_DGA" = xyes)
    AM_CONDITIONAL(ATIMISC_TV_OUT, test "x$ATIMISC_TV_OUT" = xyes)
    AM_CONDITIONAL(USE_EXA, test "x$USE_EXA" = xyes)
  ])
