dnl
dnl eg
dnl  XORG_M4_LOOP([TESTING_@1=testing], [alpha], [bravo], [charlie], [delta])
dnl
dnl This loop has a deliminated argument list
dnl
dnl first arg is the loop contents with @1 representing the active variable
dnl
dnl 1 arg
AC_DEFUN([XORG_M4_LOOP], [m4_if([$2], [], [],dnl
[m4_bpatsubst([$1], [@1], [$2]) XORG_M4_LOOP([$1], m4_shift(m4_shift($@)))])dnl
])dnl

dnl 2 arg
AC_DEFUN([XORG_M4_LOOP_2], [m4_if([$2], [], [],dnl
[m4_bpatsubst([m4_bpatsubst([$1], [@1], [$2])], [@2], [$3]) XORG_M4_LOOP_2([$1], m4_shift(m4_shift(m4_shift($@))))])dnl
])dnl

dnl
dnl eg.
dnl  XORG_M4_FOR([TESTING_@2=testing], [alpha bravo charlie delta])
dnl
dnl This loop takes a space seperated string as the list of arguments
dnl
dnl first arg is the loop contents with @1 representing the acive variable
dnl
dnl 1 arg
AC_DEFUN([XORG_M4_FOR], [XORG_M4_LOOP([$1], XORG_SFX_LST([$2], [,]))])dnl

dnl 2 arg
AC_DEFUN([XORG_M4_FOR_2], [XORG_M4_LOOP_2([$1], XORG_SFX_LST([$2],[,]))])dnl

dnl
dnl place a suffix after each word in a list
dnl
AC_DEFUN([XORG_SFX_LST],[m4_bpatsubst([$1], [[-a-z0-9A-Z_]+], [\&$2])])dnl

dnl
dnl place a prefix before each word in a list
dnl
AC_DEFUN([XORG_PFX_LST],[m4_bpatsubst([$1], [[-a-z0-9A-Z_]+], [$2\&])])dnl

dnl
dnl Capitolize the argument at compile time
dnl
AC_DEFUN([Capitols], [translit([$1], [-a-z], [_A-Z])])dnl

dnl
dnl Capitolize the argument at run time
dnl
AC_DEFUN([RTCapitols],[`echo $1 | sed 's%[[-/]]%_%g ; s%.*%\U&%g'`])

dnl
dnl  fix dashes and / to underscores at compile time
dnl
AC_DEFUN([Underscore], [translit([$1], [-/], [__])])dnl

dnl
dnl  fix dashes and / to underscores at run time
dnl
AC_DEFUN([RTUnderscore], [`echo $1 | sed 's%-%_%g'`])dnl

dnl
dnl  turn commas into spaces
dnl
AC_DEFUN([Commas2Spaces], [translit([$1], [,], [ ])])dnl

dnl
dnl  turn commas into spaces
dnl
AC_DEFUN([RTCommas2Spaces], [`echo $1 | sed 's%,% %g'`])dnl

dnl
dnl get module version at run time - LEAVE the embedded newline in translit
dnl
AC_DEFUN([XMOD_VERSION],[translit(esyscmd(cat $1/version), [
], [])])dnl

dnl
dnl CHeck for echo -e
dnl
AC_DEFUN([XORG_ECHO], [dnl
  case `echo "testing\c"; echo 1,2,3`,`echo -e testing\c; echo 1,2,3` in
    *c*,-e*c*) XECHO_E="echo"    ; ECHO_EXTEND=no  ;;
    *c*,*  )   XECHO_E="echo -e" ; ECHO_EXTEND=yes ;;
    *)         XECHO_E="echo"    ; ECHO_EXTEND=yes ;;
  esac
  if test "x$ECHO_EXTEND" = "xyes" ; then
    XYELLOW="\033@<:@1;33m"
    XRED="\033@<:@1;31m"
    XGREEN="\033@<:@1;32m"
    XBLUE="\033@<:@1;34m"
    XDEFAULT="\033@<:@0m"
  else
    XYELLOW=""
    XRED=""
    XGREEN=""
    XBLUE=""
    XDEFAULT=""
  fi
])dnl

dnl
dnl Format end of configure output
dnl
AC_DEFUN([XORG_FORMAT], [dnl
m4_format([${XYELLOW} %-20s v. %-9.9s ${XDEFAULT}],dnl
m4_if(m4_eval(m4_len($1)>20), [1],
[m4_substr([$1],[0],[11])${XRED}...${XYELLOW}dnl
m4_substr([$1],m4_eval(m4_len([$1])-6))], [$1]),XMOD_VERSION($1))])dnl

dnl
dnl Set the echo line
dnl
AC_DEFUN([XORG_OUTPUT], [AC_REQUIRE([XORG_ECHO]) dnl
  case $Underscore([XBUILD_$1]) in
    yes)  Underscore([XBUILD_$1])="${XGREEN}yes${XDEFAULT}" ;;
    no)   Underscore([XBUILD_$1])="${XRED}no ${XDEFAULT}" ;;
    sys)  Underscore([XBUILD_$1])="${XYELLOW}sys${XDEFAULT}" ;;
  esac
m4_if([$2], [], [], [dnl
  case $Underscore([XBUILD_$2]) in
    yes)  Underscore([XBUILD_$2])="${XGREEN}yes${XDEFAULT}" ;;
    no)   Underscore([XBUILD_$2])="${XRED}no ${XDEFAULT}" ;;
    sys)  Underscore([XBUILD_$2])="${XYELLOW}sys${XDEFAULT}" ;;
  esac
])dnl
  $XECHO_E "XORG_FORMAT([$1])$Underscore([XBUILD_$1])   dnl
m4_if([$2], [], ["], [XORG_FORMAT([$2])$Underscore([XBUILD_$2])"]) >> $3
])dnl

dnl
dnl Prep the results for output
dnl
AC_DEFUN([XORG_OUTPUT_RESULTS], [AC_REQUIRE([XORG_ECHO])dnl
  $XECHO_E "${XBLUE}Building $1 modules${XRESET}" > XORG.$1.build.tmp
XORG_M4_FOR_2([XORG_OUTPUT([@1], [@2], [XORG.$1.build.tmp])], [$2])dnl
])dnl

dnl
dnl  Show the results
dnl
AC_DEFUN([XORG_SHOW_RESULTS], [dnl
  for XOF in $1 ; do
    echo
    cat XORG.$XOF.build.tmp
    rm XORG.$XOF.build.tmp
  done
  echo
])dnl

dnl
dnl enable a module (yes, no, sys)
dnl
AC_DEFUN([XORG_ENABLE_MODULE], [dnl
  Underscore([XBUILD_$1])=m4_if([$2], [], [yes], [$2])
])dnl

dnl
dnl disable a module (yes, no, sys)
dnl
AC_DEFUN([XORG_DISABLE_MODULE], [dnl
  Underscore([XBUILD_$1])=no
])dnl

dnl
dnl disable a range of modules in arg 2 depending on a test at arg 1
dnl
AC_DEFUN([XORG_DISABLE_IF], [dnl
if $1 ; then
XORG_M4_FOR([XORG_DISABLE_MODULE([@1])],[$2])dnl
fi
])dnl

dnl
dnl set the prerequisites for a module
dnl
AC_DEFUN([XORG_SET_PREREQ], [dnl
XORG_M4_FOR([dnl
m4_ifdef([XPREREQ_@1_DEFINED], [], [AC_DEFUN([XPREREQ_@1_DEFINED],[])])dnl
  Underscore([XPREREQ_@1])="$Underscore([XPREREQ_@1]) $1"
], [$2])])dnl

dnl
dnl Check the prerequisites
dnl
AC_DEFUN([XORG_CHECK_PREREQ], [dnl
m4_ifdef([XPREREQ_$1_DEFINED],[dnl
if test "x$Underscore([XBUILD_$1])" = "x" ; then
  Underscore([XBUILD_$1])=yes
fi
if test "x$Underscore([XBUILD_$1])" = "xyes" ; then
  for XORG_SUBDIR in $Underscore([XPREREQ_$1]) ; do
    if  eval test "x$RTUnderscore([XBUILD_${XORG_SUBDIR}])" = "xno" ; then
      Underscore([XBUILD_$1])=no
    fi
  done
  if test "x$Underscore([XBUILD_$1])" = "xno" ; then
AC_MSG_WARN([$1 requires the modules - $Underscore([XPREREQ_$1])])
  fi
fi
])dnl
m4_if([$2], [], [], [if test x$Underscore([XBUILD_$1]) = "xyes" ; then
  $2
m4_if([$3], [], [], [elif test x$Underscore([XBUILD_$1]) = "xsys" ; then
  $3])
m4_if([$4], [], [], [else
  $3])
fi
])])dnl

dnl
dnl Check the number of arguments
dnl
AC_DEFUN([XORG_NO_ARGS], [$#])dnl

dnl
dnl
dnl
AC_DEFUN([XORG_ARGS_STRING], [XORG_NO_ARGS(XORG_SFX_LST([$1]))])dnl
dnl
dnl Check for build status of modules
dnl
AC_DEFUN([XORG_XBUILD_CHECK],[dnl
XTST=yes
ifelse(XORG_ARGS_STRING([$1]), [1], [dnl
if test "x$Underscore([XBUILD_$1])" != "xyes" && \
    test "x$Underscore([XBUILD_$1])" = "xsys" ; then
  XTST=no
fi
],[dnl
for XMOD in $1 ; do
  if eval test "x$RTUnderscore([XBUILD_$XMOD])" != "xyes" && \
      eval test "x$RTUnderscore([XBUILD_$XMOD])" != "xsys" ; then
    XTST=no
  fi
done
])
ifelse([$2], [], [dnl
ifelse([$3], [], [], [dnl
if test "x$XTST" = "xno" ; then
  $3])
],[dnl
if test "x$XTST" = "xyes" ; then
  $2
ifelse([$3], [], [], [dnl
else
  $3])
])
fi
])dnl
